// import { router } from '../../modules/router.ts';
import { runAppRouterFn } from '../app.ts';

import { Route } from '@kevisual/router';
import { RouterCodeModel, RouterCode } from '../models/code.ts';

export enum CodeStatus {
  running = 'running',
  stop = 'stop',
  fail = 'fail',
}

export type CodeManager = {
  fn?: any;
  status?: CodeStatus;
  errorMsg?: string;
  lock?: boolean; // 是否锁定
} & Partial<RouterCode>;

const codeDemoRun = `async function run(ctx) {
  ctx.body = 'test js';
  return ctx;
}`;
const templateFn = (codeStr: string) => {
  return `
  ${codeStr}
  if(run) {
    return run(ctx);
  }
  if(main) {
    return main(ctx);
  }
  return 'no run or main function';
`;
};
export const loadOne = async (item: RouterCodeModel) => {
  const { path, key, id, code, exec, project, middleware } = item.toJSON();
  const codeStr = exec || code;
  try {
    const fn: any = new Function('ctx', templateFn(codeStr));
    // run code
    const codeRunRoute = new Route(path, key, { id });
    codeRunRoute.run = fn;
    codeRunRoute.middleware = middleware;
    runAppRouterFn('removeById', id);
    runAppRouterFn('add', codeRunRoute);
    return {
      ...item.toJSON(),
      path,
      key,
      id,
      project,
      fn,
      status: CodeStatus.running,
    };
  } catch (e) {
    console.error('error id:', id, '\n', e);
    return {
      path,
      key,
      id,
      project,
      status: CodeStatus.fail,
      errorMsg: e.message.toString(),
    };
  }
};
export const load = async function () {
  const codes = await RouterCodeModel.findAll({
    logging: (sql, timing) => {
      console.log('manager load database router codeModel');
    },
  });
  const codeManager: CodeManager[] = codes.map((item) => {
    const { path, key, id, code, exec, project, active, middleware } = item.toJSON();
    if (!active) {
      return {
        ...item.toJSON(),
        path,
        key,
        id,
        code,
        project,
        middleware,
        status: CodeStatus.stop,
      };
    }
    try {
      const codeStr = exec || code;
      const fn: any = new Function('ctx', templateFn(codeStr));
      // run code
      const codeRunRoute = new Route(path, key, { id });
      codeRunRoute.run = fn;
      codeRunRoute.middleware = middleware;
      runAppRouterFn('add', codeRunRoute);
      return {
        ...item.toJSON(),
        path,
        key,
        id,
        code,
        project,
        type: item.type,
        fn,
        status: CodeStatus.running,
      };
    } catch (e) {
      console.error('error id:', id, '\n', e);
      return {
        path,
        key,
        id,
        code,
        project,
        type: item.type,
        status: CodeStatus.fail,
        errorMsg: e.message.toString(),
      };
    }
  });
  return codeManager;
};
