import stream from 'stream'; // 默认导入整个模块
const { once } = stream; // 从中解构出 EventEmitter

import { load, CodeManager, CodeStatus, loadOne } from './load.ts';
import { RouterCodeModel, TableIsExist } from '../models/code.ts';
import { emitter } from '../modules/event.ts';

export enum LoadStatus {
  LOADING = 'loading',
  LOADED = 'loaded',
  ERROR = 'error',
}
export const manager = {
  loaded: LoadStatus.LOADING, // 是否已经加载
  list: [] as CodeManager[],
  shareLocalList: [] as CodeManager[],
};

// 更新
export const updateNewCode = (code: CodeManager) => {
  const index = manager.list.findIndex((item) => item.id === code.id);
  if (index === -1) {
    manager.list.push(code);
  } else {
    manager.list[index] = code;
  }
};
// 删除
export const removeCode = (id: string) => {
  const index = manager.list.findIndex((item) => item.id === id);
  if (index !== -1) {
    manager.list.splice(index, 1);
  }
};
export const stopCode = (id: string) => {
  const index = manager.list.findIndex((item) => item.id === id);
  if (index !== -1) {
    manager.list[index].status = CodeStatus.stop;
  }
};
export const startCode = async (code: RouterCodeModel) => {
  const index = manager.list.findIndex((item) => item.id === code.id);
  console.log('index', index, code.toJSON());
  if (index !== -1) {
    manager.list[index] = await loadOne(code);
  } else {
    const codeManger = await loadOne(code);
    manager.list.push(codeManger);
  }
};

once(emitter, 'loaded')
  .then(() => {
    manager.loaded = LoadStatus.LOADED;
    console.log('manager loaded');
  })
  .catch((e) => {
    manager.loaded = LoadStatus.ERROR;
    console.error('manager loaded error', e);
  });

const init = async function () {
  const r = await load();
  manager.list = r;

  emitter.emit('loaded');
};
TableIsExist().then(async (res) => {
  if (res) {
    init();
  } else {
    console.log('TableIsExist not exist, waiting create');
    // 3s后再次检测
    setTimeout(() => {
      TableIsExist().then(async (res) => {
        if (res) {
          init();
        } else {
          console.error('TableIsExist not exist, create error');
          process.exit(1);
        }
      });
    }, 3000);
  }
});
// init();
