import { sequelize } from '../modules/sequelize.ts';
import { DataTypes, Model } from 'sequelize';

type PageNodeData = {
  id: string;
  type: string;
  data: {
    label?: string; // 容器 开始 结束
    // 容器上的属性
    cid?: string; // 容器code id
  };

  [key: string]: any;
};

export interface RouterGraphData {
  edges: any[];
  nodes: PageNodeData[];
  viewport: any;
  [key: string]: any;
}

/**
 * 页面数据
 */
export class RouterGraphModel extends Model {
  declare id: string;
  declare title: string;
  declare description: string;
  declare type: string;
  declare data: RouterGraphData;
}
RouterGraphModel.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4,
      comment: 'id',
    },
    title: {
      type: DataTypes.STRING,
      defaultValue: '',
    },
    description: {
      type: DataTypes.TEXT,
      defaultValue: '',
    },
    type: {
      type: DataTypes.STRING,
      defaultValue: '',
    },
    data: {
      type: DataTypes.JSON,
      defaultValue: {},
    },
  },
  {
    sequelize,
    tableName: 'cf_router_graph',
    paranoid: true,
  },
);
RouterGraphModel.sync({ alter: true, logging: false }).catch((e) => {
  console.error('RouterGraphModel sync', e);
});
