import { useFileStore } from '@abearxiong/use-file-store';
import { getConfigFile } from '@kevisual/use-config';
import path from 'path';
import fs from 'fs';

export const appsPath = useFileStore('apps', { needExists: true });

export type AppInfoConfig = {
  list: any[];
  [key: string]: any;
};
/**
 * 加载应用信息
 * @returns
 */
export const loadAppInfo = async (): Promise<AppInfoConfig> => {
  const pkgs = getConfigFile();
  let configFile = getConfigFile('apps.config.json');
  if (!pkgs) {
    console.error('未找到配置文件');
    return;
  }
  const basePath = path.dirname(pkgs);
  if (!configFile) {
    configFile = path.join(basePath, 'apps.config.json');
    fs.writeFileSync(configFile, JSON.stringify({}));
    return { list: [] };
  }
  const config = fs.readFileSync(configFile, 'utf-8');
  return JSON.parse(config);
};

/**
 *
 * 保存应用信息
 * @param data
 * @returns
 */
export const saveAppInfo = async (data: any) => {
  const configFile = getConfigFile('apps.config.json');
  if (!configFile) {
    return;
  }
  fs.writeFileSync(configFile, JSON.stringify(data));
};
