import { minioClient } from '@/app.ts';
import { bucketName } from '@/modules/minio.ts';
import { fileIsExist } from '@kevisual/use-config';

import fs from 'fs';
import path from 'path';
import * as tar from 'tar';
import { appsPath } from '../lib/index.ts';

export type InstallAppOpts = {
  path?: string;
  key?: string;
};
/**
 * 检测路径是否存在
 * @param opts
 * @returns
 */
export const appPathCheck = async (opts: InstallAppOpts) => {
  const { key } = opts;
  const directory = path.join(appsPath, key);
  if (fileIsExist(directory)) {
    return true;
  }
  return false;
};
export const installApp = async (opts: InstallAppOpts) => {
  const { key } = opts;
  const fileStream = await minioClient.getObject(bucketName, opts.path);
  const pathName = opts.path.split('/').pop();
  const directory = path.join(appsPath, key);
  if (!fileIsExist(directory)) {
    fs.mkdirSync(directory, { recursive: true });
  }
  const filePath = path.join(directory, pathName);

  const writeStream = fs.createWriteStream(filePath);
  fileStream.pipe(writeStream);

  await new Promise((resolve, reject) => {
    writeStream.on('finish', resolve);
    writeStream.on('error', reject);
  });
  // 解压 tgz文件
  const extractPath = path.join(directory);
  await tar.x({
    file: filePath,
    cwd: extractPath,
  });
  const pkgs = path.join(extractPath, 'package.json');
  if (!fileIsExist(pkgs)) {
    throw new Error('Invalid package.json');
  }
  const json = fs.readFileSync(pkgs, 'utf-8');
  const pkg = JSON.parse(json);
  const { name, version, app } = pkg;
  if (!name || !version || !app) {
    throw new Error('Invalid package.json');
  }
  app.key = key;
  fs.writeFileSync(pkgs, JSON.stringify(pkg, null, 2));
  // fs.unlinkSync(filePath);
  return { path: filePath, pkg };
};
